@@ -40,10 +40,11 @@ impl<R: AsyncReadExt + Unpin> H264Stream<R> {
 				return Ok(unit);
 			}
 
-			let start = self.buffer.len();
+			let mut start = self.buffer.len();
 			let read = self.reader.read_buf(&mut self.buffer).await?;
-			let end = start + read;
-			for i in start..end {
+			for i in 0..read {
+				let i = start + i;
+
 				// H264 NAL Unit Header is 0x000001 https://stackoverflow.com/a/2861340/8835688
 				if self.buffer[i] == 0x00 {
 					self.nal_unit_detect += 1;
@@ -51,7 +52,8 @@ impl<R: AsyncReadExt + Unpin> H264Stream<R> {
 				}
 
 				// Some encoder implementations write more than 2 null bytes
-				let is_nal_header = self.nal_unit_detect >= NAL_UNIT_PREFIX_NULL_BYTES && self.buffer[i] == 0x01;
+				let is_nal_header = self.nal_unit_detect >= NAL_UNIT_PREFIX_NULL_BYTES
+					&& self.buffer[i] == 0x01;
 				let nal_unit_detect = mem::replace(&mut self.nal_unit_detect, 0);
 
 				if is_nal_header {
@@ -70,6 +72,7 @@ impl<R: AsyncReadExt + Unpin> H264Stream<R> {
 
 					// Move to the start (with allocation)
 					{
+						let end = self.buffer.len();
 						let mut buffered = Vec::with_capacity(end - last_frame_end);
 						buffered.extend(&self.buffer[last_frame_end..end]);
 						self.buffer.clear();
@@ -77,6 +80,7 @@ impl<R: AsyncReadExt + Unpin> H264Stream<R> {
 					}
 
 					self.unit_buffer.push_back(H264NalUnit::new(nal_unit));
+					start = 0;
 				}
 			}
 		}
